/* Little program to attempt to extract the job current_status from the output
   of glite-job-submit and return an appropriate numerical exit code.

   The WMS documentation for glite-job-submit is wrong - this code is based
   on the contents of the glite-job-submit script itself. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define TEST_SUCCESS  0 /* must match values in testinit.sh */
#define TEST_FAILURE  1 /* must match values in testinit.sh */
#define TEST_WAIT     2 /* must match values in testinit.sh */
#define TEST_ABANDON  3 /* must match values in testinit.sh */
#define TEST_COMPLETE 4 /* internal value */

main(int argc, char *argv[])
{
	FILE	*fp;
	int	current_status,
		exit_code;
	char	*filename, 
		buf[255],
		*s;

	if (argc != 2)
	{
		fprintf(stderr, "Usage: %s <current_status-file>\n", argv[0]);
		exit(TEST_ABANDON);
	}
	filename = argv[1];

	fp = fopen(filename, "r");
	if (fp == NULL)
	{
		fprintf(stderr, "Can't open %s for read.\n", filename);
		exit(TEST_ABANDON);
	}

	current_status = TEST_ABANDON; /* default */
	while (fgets(buf, sizeof(buf), fp))
	{
		s = strchr(buf, '\n');
		if (s == NULL)
		{
			fprintf(stderr, "Input line overflows buffer.\n");
			exit(TEST_ABANDON);
		}
		*s = '\0';

		for (s = buf; *s != '\0'; ++s) *s = toupper(*s);

		if (strstr(buf, "CURRENT STATUS:"))
		{
			if (strstr(buf, "DONE (SUCCESS)"))
			{
				current_status = TEST_COMPLETE;
			}
			else if (strstr(buf, "SUBMITTED") ||
                                 strstr(buf, "WAITING") ||
			         strstr(buf, "READY") ||
			         strstr(buf, "SCHEDULED") ||
			         strstr(buf, "RUNNING"))
			{
				exit(TEST_WAIT);
			}
			else /* many and various, mostly undocumented */
			{
				exit(TEST_ABANDON);
			}

		}
		else if (current_status == TEST_COMPLETE &&
		         strstr(buf, "EXIT CODE:"))
		{
			s = strchr(buf, ':') + 1;
			exit_code = atoi(s);
			if (exit_code == 0)
			{
				exit(TEST_SUCCESS);
			}
			else
			{
				exit(TEST_FAILURE);
			}
		}
	}

	fprintf(stderr, "Can't find status in status file.\n");
	exit(TEST_ABANDON);
}

/* End of file. */
