#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include "rgma.h"

int main(int argc, char *argv[])
{ 
    RGMAResource *r;
    RGMAResourceEndpoint *endpoint;
    RGMAResultSet *rs;
    char *testId, *tableName, *queryTypeS, select[255];
    int terminationInterval, queryTimeout, numTuples, queryType,
        oldness, startTime, n, endOfResults;

    /* Check arguments. */

    if (argc != 7)
    {
	fprintf(stderr, "Usage: <testId> <tableName> <terminationInterval> <queryTimeout> <queryType> <numTuples>\n");
        exit(1);
    }

    testId = argv[1];
    tableName = argv[2];
    terminationInterval = atoi(argv[3]); //seconds
    queryTimeout = atoi(argv[4]); //seconds
    queryTypeS = argv[5];
    numTuples = atoi(argv[6]);

    /* Create Consumer to consume from tableName. Do a continuous+old
       query in case the producer manages to start up more quickly than
       the consumer. */

    startTime = time(NULL);

    sprintf(select, "SELECT * FROM %s WHERE aString = '%s'", tableName, testId);

    if (strcmp(queryTypeS, "CONTINUOUS") == 0)
    {
        oldness = queryTimeout;
        queryType = RGMAQueryType_CONTINUOUS | RGMAQueryType_INTERVAL;
    }
    else if (strcmp(queryTypeS, "LATEST") == 0)
    {
        oldness = 0;
        queryType = RGMAQueryType_LATEST;
    }
    else if (strcmp(queryTypeS, "HISTORY") == 0)
    {
        oldness = 0;
        queryType = RGMAQueryType_HISTORY;
    }
    else
    {
        fprintf(stderr, "Invalid query type (%s) in test script.\n", queryType);
        exit(1);
    }

    if (RGMA_createConsumer(terminationInterval, select, queryType, oldness, &r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    endpoint = RGMA_getEndpoint(r);
    if (endpoint == NULL)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    if (RGMAConsumer_start(r, queryTimeout) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    /* Wait for tuples: since we're timing things, we ought to have a short
       timeout in the sleep statement. NB. Need to ensure that pop() gets
       called during AND after the execution of the query. */

    n = 0; /* total number of tuples */
    endOfResults = 0;
    if (strcmp(queryTypeS, "CONTINUOUS") == 0) sleep(queryTimeout);
    while(!endOfResults)
    {
        endOfResults = (!RGMAConsumer_isExecuting(r));

        rs = RGMAConsumer_pop(r, numTuples);
        if (rs == NULL)
        {
            fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
            exit(1);
        }
        n += rs->numRows;
	RGMA_freeResultSet(rs);

        sleep(1);
    }
    
    if (n != numTuples)
    {
        fprintf(stderr, "Expected %d tuples, got %d.\n", numTuples, n);
        exit(1);
    }
    
    printf("URL=%s\n", endpoint->url);
    printf("resourceId=%d\n", endpoint->resourceId);
    printf("startTime=%d\n", startTime);
    printf("endTime=%d\n", time(NULL));

    /* Clean up. */

    RGMA_close(r);
    RGMA_freeEndpoint(endpoint);
    RGMA_freeResource(r);
    exit(0);
}

/* End of file. */
