#include "impl/ConsumerFactoryImpl.h"

#include <string>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

using rgma::impl::ConsumerFactoryImpl;
using glite::rgma::ConsumerFactory;
using glite::rgma::Consumer;
using glite::rgma::QueryProperties;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::ResultSet;
using glite::rgma::ResourceEndpoint;
using glite::rgma::ExceptionBase;

int main(int argc, char *argv[])
{ 
    /* Check arguments. */

    if (argc != 7)
    {
        std::cerr << "Usage: <testId> <tableName> <terminationInterval> <queryTimeout> <queryType> <numTuples>\n" << std::endl;
        exit(1);
    }
    std::string testId(argv[1]);
    std::string tableName(argv[2]);
    int terminationInterval = atoi(argv[3]); //seconds
    int queryTimeout = atoi(argv[4]); //seconds
    std::string queryType(argv[5]);
    int numTuples = atoi(argv[6]);

    /* Create Consumer to consume from tableName. Do a continuous+old
       query in case the producer manages to start up more quickly than
       the consumer. */

    int startTime = time(NULL);
    ResourceEndpoint endpoint;

    ConsumerFactory *cfactory = new ConsumerFactoryImpl();
    Consumer *c;

    try
    {
        std::string select;
        select.append("SELECT * FROM ");
        select.append(tableName);
        select.append(" WHERE aString = '" + testId + "'");
        TimeInterval terminationInterval(terminationInterval, Units::SECONDS);
        if (queryType == "CONTINUOUS")
        {
            TimeInterval oldness(queryTimeout, Units::SECONDS);
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::getContinuous(oldness));
        }
        else if (queryType == "LATEST")
        {
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::LATEST);
        }
        else if (queryType == "HISTORY")
        {
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::HISTORY);
        }
        else
        {
            std::cerr << "Invalid query type (" << queryType << ") in test script.\n";
            exit(1);
        }

        endpoint = c->getEndpoint();

        c->start(TimeInterval(queryTimeout, Units::SECONDS));
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to create/start consumer."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    /* Wait for tuples: since we're timing things, we ought to have a short
       timeout in the sleep statement. NB. Need to ensure that pop() gets
       called during AND after the execution of the query. */

    try
    {
        int n = 0;
        int endOfResults = 0;
        if (queryType == "CONTINUOUS") sleep(queryTimeout);
        while(!endOfResults)
        {
            endOfResults = (!c->isExecuting());

            ResultSet resultSet;
            c->pop(resultSet, numTuples);
            ResultSet::iterator i;
            for (i = resultSet.begin(); i < resultSet.end(); ++i)
            {
                ++n;
            }

            sleep(1);
        }

        if (n != numTuples)
        {
            std::cerr << "Expected " << numTuples << " tuples, got " << n << "." << std::endl;
            exit(1);
        }
    
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to retrieve results."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    std::cout << "URL=" << endpoint.getUrl() << "\n";
    std::cout << "resourceId=" << endpoint.getResourceId() << "\n";
    std::cout << "startTime=" << startTime << "\n";
    std::cout << "endTime=" << time(NULL) << "\n";

    /* Clean up. */

    c->close();
    delete cfactory;
    exit(0);
}

/* End of file. */
