import org.glite.rgma.*;
import org.glite.rgma.stubs.ConsumerFactoryStub;

public class test_c
{
    public static void main(String[] args)
    { 
        /* Check arguments. */
    
        if (args.length != 6)
        {
            System.err.println("Usage: <testId> <tableName> <terminationInterval> <queryTimeout> <queryType> <numTuples>");
            System.exit(1);
        }
    
        String testId = args[0];
        String tableName = args[1];
        int terminationInterval = Integer.parseInt(args[2]); //seconds
        int queryTimeout = Integer.parseInt(args[3]); //seconds
        String queryType = args[4];
        int numTuples = Integer.parseInt(args[5]);
    
        /* Create Consumer to consume from tableName. Do a continuous+old
           query in case the producer manages to start up more quickly than
           the consumer. */
    
        long startTime = System.currentTimeMillis() / 1000;
    
	String select = "SELECT * FROM " + tableName + " WHERE aString = '" +
                        testId + "'";
	QueryProperties queryProperties = null;

        if (queryType.equals("CONTINUOUS"))
        {
            TimeInterval oldness = new TimeInterval(queryTimeout, Units.SECONDS);
            queryProperties = QueryProperties.getContinuous(oldness);
        }
        else if (queryType.equals("LATEST"))
        {
            queryProperties = QueryProperties.LATEST;
        }
        else if (queryType.equals("HISTORY"))
        {
            queryProperties = QueryProperties.HISTORY;
        }
        else
        {
            System.err.println("Invalid query type (" + queryType + ") in test script.");
            System.exit(1);
        }

        Consumer c = null;
        ResourceEndpoint endpoint = null;

        try
        {
	    ConsumerFactory cf = new ConsumerFactoryStub();

            c = cf.createConsumer(new TimeInterval(terminationInterval, Units.SECONDS), select, queryProperties);
            endpoint = c.getResourceEndpoint();
            c.start(new TimeInterval(queryTimeout, Units.SECONDS));
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            System.exit(1);
        }
    
        /* Wait for tuples: since we're timing things, we ought to have a short
           timeout in the sleep statement. NB. Need to ensure that pop() gets
           called during AND after the execution of the query. */
    
        int n = 0; // total number of tuples
        try
        {
            boolean endOfResults = false;
            if (queryType.equals("CONTINUOUS")) Thread.sleep(queryTimeout * 1000);
            while (!endOfResults)
            {
                endOfResults = (!c.isExecuting());

                ResultSet rs = c.pop(numTuples);
                while (rs.next()) ++n;

                Thread.sleep(1000);
            }
            
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            System.exit(1);
        }
            
        if (n != numTuples)
        {
            System.err.println("Expected " + numTuples + " tuples, got " + n + ".");
            System.exit(1);
        }
        
        System.out.println("URL=" + endpoint.getUrl());
        System.out.println("resourceId=" +  endpoint.getResourceId());
        System.out.println("startTime=" + startTime);
        System.out.println("endTime=" + (System.currentTimeMillis() / 1000));
    
        /* Clean up. */
    
        try {c.close();} catch (Exception e) {}; // ignore any errors
        System.exit(0);
    }
}
    
/* End of file. */
