#!/usr/bin/env python

import rgma
import sys
import time

# Define main() function.

def main():

    # Check arguments.

    if len(sys.argv) != 7:
        sys.stderr.write("Usage: <testId> <tableName> <terminationInterval> <queryTimeout> <queryType> <numTuples>\n")
        sys.exit(1)

    testId = sys.argv[1]
    tableName = sys.argv[2]
    terminationInterval = sys.argv[3] #seconds
    queryTimeout = sys.argv[4] #seconds
    queryType = sys.argv[5]
    numTuples = sys.argv[6]

    # Create Consumer to consume from tableName. Do a continuous+old
    # query in case the producer manages to start up more quickly than
    # the consumer.

    startTime = time.time()

    ti = rgma.TimeInterval(terminationInterval, rgma.Units.SECONDS)

    select = "SELECT * FROM " + tableName + " WHERE aString = '" + testId + "'"

    if queryType == "CONTINUOUS":
        timeInterval = rgma.TimeInterval(queryTimeout, rgma.Units.SECONDS)
        queryProperties = rgma.QueryProperties(rgma.QueryType.CONTINUOUS,
                                               timeInterval)
    elif queryType == "LATEST":
        queryProperties = rgma.QueryProperties(rgma.QueryType.LATEST)
    elif queryType == "HISTORY":
        queryProperties = rgma.QueryProperties(rgma.QueryType.HISTORY)
    else:
        sys.stderr.write("Invalid query type (" + queryType + ") in test script.\n")
        sys.exit(1)

    try:
        c = rgma.Consumer(ti, queryProperties, select)
        endpoint = c.getEndpoint()
        c.start(rgma.TimeInterval(queryTimeout, rgma.Units.SECONDS))
    except Exception, e:
        sys.stderr.write(e.__str__() + "\n")
        sys.exit(1)
    
    # Wait for tuples: since we're timing things, we ought to have a short
    # timeout in the sleep statement. NB. Need to ensure that pop() gets
    # called during AND after the execution of the query.
    
    n = 0 # total number of tuples
    try:
        endOfResults = False
        if queryType == "CONTINUOUS": time.sleep(int(queryTimeout))
        while not endOfResults:
            endOfResults = not c.isExecuting()

            rs = c.pop(numTuples)
            for row in rs: n = n + 1

            time.sleep(1)
    except Exception, e:
        sys.stderr.write(e.__str__() + "\n")
        sys.exit(1)
            
    if n != int(numTuples):
        sys.stderr.write("Expected " + numTuples + " tuples, got " + `n` + "\n.")
#        sys.exit(1)
        
    print "URL=%s" % endpoint.getUrl()
    print "resourceId=%d" % int(endpoint.getResourceId())
    print "startTime=%.0f" % startTime
    print "endTime=%.0f" % time.time()
    
    # Clean up.
    
    try: c.close()
    except: pass # ignore any errors
    sys.exit(0)

# Run main() function.

main()

# End of file.
