#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "rgma.h"

int main(int argc, char *argv[])
{ 
    RGMAResource *r;
    RGMAResourceEndpoint *endpoint;
    char *testId, *tableName, *producerType, predicate[255], insert[255];
    int terminationInterval, historyRP, latestRP, firstTupleId, lastTupleId,
        producerProps, storageType, startTime, i;

    /* Check arguments. */

    if (argc != 9)
    {
        fprintf(stderr, "Usage: <testId> <tableName> <terminationInterval> <historyRP> <latestRP> <producerType> <firstTupleId> <lastTupleId>\n");
        exit(1);
    }

    testId = argv[1];
    tableName = argv[2];
    terminationInterval = atoi(argv[3]); //seconds
    historyRP = atoi(argv[4]); //seconds
    latestRP = atoi(argv[5]); //seconds
    producerType = argv[6];
    firstTupleId = atoi(argv[7]);
    lastTupleId = atoi(argv[8]);

    /* Create Primary Producer and insert numRows rows into tableName, with
       separate INSERTs. */

    startTime = time(NULL);

    if (strcmp(producerType, "CONTINUOUS") == 0)
    {
        producerProps = 0;
	storageType = RGMAStorageType_MEMORY;
    }
    else if (strcmp(producerType, "LATEST") == 0)
    {
        producerProps = RGMAProducerProps_LATEST;
	storageType = RGMAStorageType_DATABASE;
    }
    else if (strcmp(producerType, "HISTORY") == 0)
    {
        producerProps = RGMAProducerProps_HISTORY;
	storageType = RGMAStorageType_DATABASE;
    }
    else
    {
        fprintf(stderr, "Invalid producer type (%s) in test script.\n", producerType);
        exit(1);
    }

    if (RGMA_createPrimaryProducer(terminationInterval, producerProps,
                                   storageType, NULL, 0, NULL, &r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    endpoint = RGMA_getEndpoint(r);
    if (endpoint == NULL)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    sprintf(predicate, "WHERE aString = '%s'", testId);
    if (RGMAPrimaryProducer_declareTable(r, tableName, predicate, historyRP,
                                         latestRP) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    for (i = firstTupleId; i <= lastTupleId; ++i)
    {
        sprintf(insert, "INSERT INTO %s (userId, aString, aReal, anInt) VALUES ('%s-%03d','%s',0,%ld)", tableName, testId, i, testId, time(NULL));

        if (RGMAPrimaryProducer_insert(r, insert) != 0)
        {
            fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
            exit(1);
        }
    }

    printf("URL=%s\n", endpoint->url);
    printf("resourceId=%d\n", endpoint->resourceId);
    printf("startTime=%d\n", startTime);
    printf("endTime=%d\n", time(NULL));

    /* Clean up. */

    /*RGMA_close(r); leave producer to expire on its own */
    RGMA_freeResource(r);
    RGMA_freeEndpoint(endpoint);
    exit(0);
}

/* End of file. */
