#include "impl/ProducerFactoryImpl.h"

#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

using rgma::impl::ProducerFactoryImpl;
using glite::rgma::ProducerFactory;
using glite::rgma::PrimaryProducer;
using glite::rgma::ProducerProperties;
using glite::rgma::Storage;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::ResourceEndpoint;
using glite::rgma::ExceptionBase;

int main(int argc, char *argv[])
{ 
    /* Check arguments. */

    if (argc != 9)
    {
        std::cerr << "Usage: <testId> <tableName> <terminationInterval> <historyRP> <latestRP> <producerType> <firstTupleId> <lastTupleId>\n" << std::endl;
        exit(1);
    }
    std::string testId(argv[1]);
    std::string tableName(argv[2]);
    int terminationInterval = atoi(argv[3]); //seconds
    int historyRP = atoi(argv[4]); //seconds
    int latestRP = atoi(argv[5]); //seconds
    std::string producerType(argv[6]);
    int firstTupleId = atoi(argv[7]);
    int lastTupleId = atoi(argv[8]);

    /* Create Primary Producer and insert numRows rows into tableName, with
       separate INSERTs. */

    int startTime = time(NULL);
    ResourceEndpoint endpoint;

    ProducerFactory *pfactory = new ProducerFactoryImpl();
    PrimaryProducer *pp;

    try
    {
        TimeInterval ti(terminationInterval, Units::SECONDS);
        StringList voNames;

        if (producerType == "CONTINUOUS")
        {
            ProducerProperties props(Storage::MEMORY);
            pp = pfactory->createPrimaryProducer(ti, props, voNames);
        }
        else if (producerType == "LATEST")
        {
            ProducerProperties props(Storage::DATABASE,
                                     ProducerProperties::LATEST);
            pp = pfactory->createPrimaryProducer(ti, props, voNames);
        }
        else if (producerType == "HISTORY")
        {
            ProducerProperties props(Storage::DATABASE,
                                     ProducerProperties::HISTORY);
            pp = pfactory->createPrimaryProducer(ti, props, voNames);
        }
        else
        {
            std::cerr << "Invalid producer type (" << producerType << ") in test script.\n";
            exit(1);
        }

	endpoint = pp->getEndpoint();
    
        std::string predicate("WHERE aString = '" + testId + "'");
        TimeInterval hrp(historyRP, Units::SECONDS);
        TimeInterval lrp(latestRP, Units::SECONDS);
        pp->declareTable(tableName,  predicate, hrp, lrp);
    
	char insertBuf[256];
        std::string insert(insertBuf);
	for (int i = firstTupleId; i <= lastTupleId; ++i)
	{
		sprintf(insertBuf, "INSERT INTO %s (userId, aString, aReal, anInt) VALUES ('%s-%03d','%s',0,%ld)", tableName.c_str(), testId.c_str(), i, testId.c_str(), time(NULL));
        	pp->insert(insertBuf); 
	}
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to create producer and insert data."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    std::cout << "URL=" << endpoint.getUrl() << "\n";
    std::cout << "resourceId=" << endpoint.getResourceId() << "\n";
    std::cout << "startTime=" << startTime << "\n";
    std::cout << "endTime=" << time(NULL) << "\n";

    /* Clean up. */

    //pp->close(); // leave producer to expire on its own
    delete pfactory;
    exit(0);
}

/* End of file. */
