import org.glite.rgma.*;
import org.glite.rgma.stubs.ProducerFactoryStub;

public class test_pp
{
    public static void main(String[] args)
    { 
        /* Check arguments. */
    
        if (args.length != 8)
        {
            System.err.println("Usage: <testId> <tableName> <terminationInterval> <historyRP> <latestRP> <producerType> <firstTupleId> <lastTupleId>");
            System.exit(1);
        }
    
        String testId = args[0];
        String tableName = args[1];
        int terminationInterval = Integer.parseInt(args[2]); //seconds
        int historyRP = Integer.parseInt(args[3]); //seconds
        int latestRP = Integer.parseInt(args[4]); //seconds
        String producerType = args[5];
        int firstTupleId = Integer.parseInt(args[6]);
        int lastTupleId = Integer.parseInt(args[7]);
    
        /* Create Primary Producer and insert numRows rows into tableName, with
           separate INSERTs. */
    
        long startTime = System.currentTimeMillis() / 1000;
        ProducerProperties producerProps = null;
    
        if (producerType.equals("CONTINUOUS"))
        {
            producerProps = new ProducerProperties(Storage.MEMORY, 0);
        }
        else if (producerType.equals("LATEST"))
        {
            producerProps = new ProducerProperties(Storage.DATABASE, ProducerProperties.LATEST);
        }
        else if (producerType.equals("HISTORY"))
        {
            producerProps = new ProducerProperties(Storage.DATABASE, ProducerProperties.HISTORY);
        }
        else
        {
            System.err.println("Invalid producer type (" + producerType + ") in test script.");
            System.exit(1);
        }

        PrimaryProducer pp = null;
        ResourceEndpoint endpoint = null;
    
        try
        {
            ProducerFactory pf = new ProducerFactoryStub();
            TimeInterval ti = new TimeInterval(terminationInterval, Units.SECONDS);
            pp = pf.createPrimaryProducer(ti, producerProps, null);
            endpoint = pp.getResourceEndpoint();
    
            String predicate = "WHERE aString = '" + testId + "'";
            pp.declareTable(tableName,
                           predicate,
                           new TimeInterval(historyRP, Units.SECONDS),
                           new TimeInterval(latestRP, Units.SECONDS));
    
            String insert = null;
            for (int i = firstTupleId; i <= lastTupleId; ++i)
            {
                insert = "INSERT INTO " + tableName + " (userId, aString, aReal, anInt) VALUES ('" + testId + "-" + i + "','" + testId + "',0," + (System.currentTimeMillis() / 1000 + ")");
                pp.insert(insert);
            }
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage()); 
            System.exit(1);
        }
    
        System.out.println("URL=" + endpoint.getUrl());
        System.out.println("resourceId=" + endpoint.getResourceId());
        System.out.println("startTime=" + startTime);
        System.out.println("endTime=" + (System.currentTimeMillis() / 1000));
    
        /* Clean up. */
    
	/* pp.close(); leave producer to expire on its own */
        System.exit(0);
    }
}
    
/* End of file. */
