import java.util.Date;
import java.text.SimpleDateFormat;

import org.glite.rgma.*;
import org.glite.rgma.stubs.ConsumerFactoryStub;

public class test_select
{
    public static void main(String[] args)
    { 
        /* Check arguments. */
    
        if (args.length != 4)
        {
            System.err.println("Usage: <terminationInterval> <queryTimeout> <queryType> <select>");
            System.exit(1);
        }
    
        int terminationInterval = Integer.parseInt(args[0]); //seconds
        int queryTimeout = Integer.parseInt(args[1]); //seconds
        String queryType = args[2];
        String select = args[3];
    
        /* Create Consumer to consume from tableName. Do a continuous+old
           query in case the producer manages to start up more quickly than
           the consumer. */
    
        long startTime = System.currentTimeMillis() / 1000;
    
        QueryProperties queryProperties = null;

        if (queryType.equals("CONTINUOUS"))
        {
            TimeInterval oldness = new TimeInterval(queryTimeout, Units.SECONDS);
            queryProperties = QueryProperties.getContinuous(oldness);
        }
        else if (queryType.equals("LATEST"))
        {
            queryProperties = QueryProperties.LATEST;
        }
        else if (queryType.equals("HISTORY"))
        {
            queryProperties = QueryProperties.HISTORY;
        }
        else
        {
            System.err.println("Invalid query type (" + queryType + ") in test script.");
            System.exit(1);
        }

        Consumer c = null;
        ResourceEndpoint endpoint = null;

        try
        {
            ConsumerFactory cf = new ConsumerFactoryStub();

            c = cf.createConsumer(new TimeInterval(terminationInterval, Units.SECONDS),
                                  select, queryProperties);
            endpoint = c.getResourceEndpoint();
            c.start(new TimeInterval(queryTimeout, Units.SECONDS));
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            System.exit(1);
        }
    
        /* Wait for tuples: since we're timing things, we ought to have a short
           timeout in the sleep statement. */

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
        String timestamp;
        Date date;
        String measurementDate;
        String measurementTime;

        int numTuples = 0;
        long maxTupleAge = 0;
        long currentTime = System.currentTimeMillis();
        long tupleAge;
    
        try
        {
            boolean endOfResults = false;
            if (queryType.equals("CONTINUOUS")) Thread.sleep(queryTimeout * 1000);
            while(!endOfResults)
            {
                endOfResults = (!c.isExecuting());
                ResultSet rs = c.pop(1000);
                while (rs.next())
                {
                    ++numTuples;
                    measurementDate = rs.getString("MeasurementDate");
                    measurementTime = rs.getString("MeasurementTime");
                    timestamp = measurementDate + " " + measurementTime + " UTC";
                    date = sdf.parse(timestamp);
                    tupleAge = (currentTime - date.getTime()) / 1000; // seconds
                    if (tupleAge > maxTupleAge) maxTupleAge = tupleAge;
                }

                Thread.sleep(1000);
            }
            
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            System.exit(1);
        }
            
        System.out.println("URL=" + endpoint.getUrl());
        System.out.println("resourceId=" +  endpoint.getResourceId());
        System.out.println("startTime=" + startTime);
        System.out.println("endTime=" + (System.currentTimeMillis() / 1000));
        System.out.println("numTuples=" + numTuples);
        System.out.println("maxTupleAge=" + maxTupleAge);
    
        /* Clean up. */
    
        try {c.close();} catch (Exception e) {}; // ignore any errors
        System.exit(0);
    }
}

/* End of file. */
