#!/usr/bin/env python
                                                                                
import rgma
import sys
import time
from time import strptime

# Cheap and cheerful function to calculate offset in seconds since the
# epoch (00:00:00 UTC, January 1, 1970). This is only for
# comparing tuple times for testing purposes (which will be recent and
# in the same or consecutive years), so this code is good enough.

def sec_since_1970(date):

    DAYS = [31,28,31,30,31,30,31,31,30,31,30,31]

    year = date.tm_year
    mon = date.tm_mon - 1   # make it in the range 0 - 11
    mday = date.tm_mday - 1 # make it in the range 0 - 30
    hour = date.tm_hour
    min = date.tm_min
    sec = date.tm_sec

    if (year < 1970 or mon < 0 or mon > 11 or mday < 0 or mday > 30 or \
        hour < 0 or hour > 23 or min < 0 or min > 59 or sec < 0 or sec > 61):
        return -1;

    days = 0

    for y in range(1970, year):
        if (y % 4 == 0 and (y % 100 != 0 or y % 400 == 0)): days += 366
        else: days += 365;

    for m in range(0, mon):
        days += DAYS[m];

    if (mon > 1): # month 1 is February
        if (year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)): days += 1
        
    days += mday

    return (days * 24 * 60 * 60 + hour * 60 * 60 + min * 60 + sec)

# Little function to extract the value of the requested column from the
# requested row of a result set - in gLite1.5 you can get it directly.

def getResultSetValue(rs, row, columnName):

    return row[rs.columnNames.index(columnName)]

# Define main function.
                                                                                
def main():
                                                                                
    # Check arguments.
                                                                                
    if len(sys.argv) != 5:
        sys.stderr.write("Usage: <terminationInterval> <queryTimeout> <queryType> <select>\n")
        sys.exit(1)
                                                                                
    terminationInterval = sys.argv[1] #seconds
    queryTimeout = sys.argv[2] #seconds
    queryType = sys.argv[3]
    select = sys.argv[4]
    
    # Create Consumer to consume from tableName. Do a continuous+old
    # query in case the producer manages to start up more quickly than
    # the consumer.
    
    startTime = time.time()
                                                                                
    ti = rgma.TimeInterval(terminationInterval, rgma.Units.SECONDS)

    if queryType == "CONTINUOUS":
        oldness = rgma.TimeInterval(queryTimeout, rgma.Units.SECONDS)
        queryProperties = rgma.QueryProperties(rgma.QueryType.CONTINUOUS,
                                               oldness)

    elif queryType == "LATEST":
        queryProperties = rgma.QueryProperties(rgma.QueryType.LATEST)
    elif queryType == "HISTORY":
        queryProperties = rgma.QueryProperties(rgma.QueryType.HISTORY)
    else:
        sys.stderr.write("Invalid query type (" + queryType + ") in test script.")
        sys.exit(1)

    try:
        c = rgma.Consumer(ti, queryProperties, select)
        endpoint = c.getEndpoint()
        c.start(rgma.TimeInterval(queryTimeout, rgma.Units.SECONDS))
    except Exception, e:
        sys.stderr.write(e.__str__() + "\n")
        sys.exit(1)

    # Wait for tuples: since we're timing things, we ought to have a short
    # timeout in the sleep statement.

    numTuples = 0
    maxTupleAge = 0
    currentTime = int(time.time())

    try:
        endOfResults = False
        if queryType == "CONTINUOUS": time.sleep(int(queryTimeout))
        while not endOfResults:
            endOfResults = not c.isExecuting()
                                                                                
            rs = c.pop(1000)
            for row in rs:
                numTuples += 1
                measurementDate = getResultSetValue(rs, row, "MeasurementDate")
                measurementTime = getResultSetValue(rs, row, "MeasurementTime")
                timestamp = measurementDate + " " + measurementTime
                date = strptime(timestamp, "%Y-%m-%d %H:%M:%S")
                tupleAge = currentTime - sec_since_1970(date)
                if tupleAge > maxTupleAge: maxTupleAge = tupleAge

                                                                                
            time.sleep(1)
    except Exception, e:
        sys.stderr.write(e.__str__() + "\n")
        sys.exit(1)

    print "URL=%s" % endpoint.getUrl()
    print "resourceId=%d" % int(endpoint.getResourceId())
    print "startTime=%.0f" % startTime
    print "endTime=%.0f" % time.time()
    print "numTuples=%d" % numTuples
    print "maxTupleAge=%d" % maxTupleAge

    # Clean up.
                                                                                
    try: c.close()
    except: pass # ignore any errors
    sys.exit(0)
                                                                                
# Run main() function.
                                                                                
main()
                                                                                
# End of file.
