/* Little program to re-connect to an existing resource and send a
   showSignOfLife to it. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rgma.h"

int main(int argc, char *argv[])
{ 
    RGMAResource *r;
    RGMAResourceEndpoint endpoint;
    char *resourceTypeS;
    int resourceType;

    /* Check arguments. */

    if (argc != 4)
    {
        fprintf(stderr, "Usage: <URL> <resourceId> <PrimaryProducer|SecondaryProducer|Consumer>\n");
        exit(1);
    }

    endpoint.url = argv[1];
    endpoint.resourceId = atoi(argv[2]);
    resourceTypeS = argv[3];

    /* Attempt to re-create resource from endpoint and send showSignOfLife. */

    if (strcmp(resourceTypeS, "PrimaryProducer") == 0)
    {
        resourceType = RGMAResourceType_PRIMARYPRODUCER;
    }
    else if (strcmp(resourceTypeS, "SecondaryProducer") == 0)
    {
        resourceType = RGMAResourceType_SECONDARYPRODUCER;
    }
    else if (strcmp(resourceTypeS, "Consumer") == 0)
    {
        resourceType = RGMAResourceType_CONSUMER;
    }
    else
    {
        fprintf(stderr, "Invalid resource type (%s) in test script.\n", resourceType);
        exit(1);
    }

    /* Reconnect to resource and send a showSignOfLife. */

    if (RGMA_reconnect(&endpoint, resourceType, &r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    if (RGMA_showSignOfLife(r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    /* Clean up. */

    exit(0);
}

/* End of file. */
