/* Little program to re-connect to an existing resource and send a
   showSignOfLife to it. */

#include "impl/ConsumerFactoryImpl.h"
#include "impl/ProducerFactoryImpl.h"

#include <iostream>
#include <stdlib.h>

using rgma::impl::ProducerFactoryImpl;
using rgma::impl::ConsumerFactoryImpl;
using glite::rgma::ProducerFactory;
using glite::rgma::ConsumerFactory;
using glite::rgma::PrimaryProducer;
using glite::rgma::Consumer;
using glite::rgma::ResourceEndpoint;
using glite::rgma::ExceptionBase;

int main(int argc, char *argv[])
{ 
    /* Check arguments. */

    if (argc != 4)
    {
        std::cerr << "Usage: <URL> <resourceId> <PrimaryProducer|SecondaryProducer|Consumer>\n" << std::endl;
        exit(1);
    }

    std::string url(argv[1]);
    int resourceId = atoi(argv[2]);
    std::string resourceType(argv[3]);

    /* Attempt to re-create resource from endpoint and send showSignOfLife. */

    try
    {
        if (resourceType == "PrimaryProducer")
        {
            ProducerFactory *pfactory = new ProducerFactoryImpl();
            ResourceEndpoint endpoint(url, resourceId);
            PrimaryProducer *pp = pfactory->reconnectPrimaryProducer(endpoint);
            pp->showSignOfLife();
            delete pfactory;
        }
        else if (resourceType == "SecondaryProducer")
        {
            ProducerFactory *pfactory = new ProducerFactoryImpl();
            ResourceEndpoint endpoint(url, resourceId);
            SecondaryProducer *sp = pfactory->reconnectSecondaryProducer(endpoint);
            sp->showSignOfLife();
            delete pfactory;
        }
        else if (resourceType == "Consumer")
        {
            ConsumerFactory *cfactory = new ConsumerFactoryImpl();
            ResourceEndpoint endpoint(url, resourceId);
            Consumer *c = cfactory->reconnectConsumer(endpoint);
            c->showSignOfLife();
            delete cfactory;
        }
        else
        {
            std::cerr << "Invalid resource type (" << resourceType << ") in test script.\n";
            exit(1);
        }
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to create reconnect to resource."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    /* Clean up. */

    exit(0);
}

/* End of file. */
