#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "rgma.h"

int main(int argc, char *argv[])
{ 
    RGMAResource *r;
    RGMAResourceEndpoint *endpoint;
    char *testId, *tableName, *producerType;
    int terminationInterval, historyRP, producerProps, storageType,
        startTime;

    /* Check arguments. */

    if (argc != 6)
    {
        fprintf(stderr, "Usage: <testId> <tableName> <terminationInterval> <historyRP> <producerType>\n");
        exit(1);
    }

    testId = argv[1];
    tableName = argv[2];
    terminationInterval = atoi(argv[3]); //seconds
    historyRP = atoi(argv[4]); //seconds
    producerType = argv[5];

    /* Create Secondary Producer. */

    startTime = time(NULL);

    if (strcmp(producerType, "LATEST") == 0)
    {
        producerProps = RGMAProducerProps_LATEST;
	storageType = RGMAStorageType_DATABASE;
    }
    else if (strcmp(producerType, "HISTORY") == 0)
    {
        producerProps = RGMAProducerProps_HISTORY;
	storageType = RGMAStorageType_DATABASE;
    }
    else
    {
        fprintf(stderr, "Invalid producer type (%s) in test script.\n", producerType);
        exit(1);
    }

    if (RGMA_createSecondaryProducer(terminationInterval, producerProps,
                                     storageType, NULL, 0, NULL, &r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    endpoint = RGMA_getEndpoint(r);
    if (endpoint == NULL)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    /* (predicate must be empty for a Secondary Producer) */

    if (RGMASecondaryProducer_declareTable(r, tableName, "", historyRP) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    printf("URL=%s\n", endpoint->url);
    printf("resourceId=%d\n", endpoint->resourceId);
    printf("startTime=%d\n", startTime);
    printf("endTime=%d\n", time(NULL));

    /* Clean up. */

    /*RGMA_close(r); leave producer to expire on its own */
    RGMA_freeResource(r);
    RGMA_freeEndpoint(endpoint);
    exit(0);
}

/* End of file. */
