#include "impl/ProducerFactoryImpl.h"

#include <string>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>

using glite::rgma::ProducerFactory;
using rgma::impl::ProducerFactoryImpl;
using glite::rgma::SecondaryProducer;
using glite::rgma::ProducerProperties;
using glite::rgma::Storage;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::ExceptionBase;

int main(int argc, char *argv[])
{ 
    /* Check arguments. */

    if (argc != 6)
    {
        std::cerr << "Usage: <testId> <tableName> <terminationInterval> <historyRP> <producerType>\n" << std::endl;
        exit(1);
    }
    std::string testId(argv[1]);
    std::string tableName(argv[2]);
    int terminationInterval = atoi(argv[3]); //seconds
    int historyRP = atoi(argv[4]); //seconds
    std::string producerType(argv[5]);

    /* Create Secondary Producer to republish tableName. */

    int startTime = time(NULL);
    ResourceEndpoint endpoint;

    ProducerFactory *pfactory = new ProducerFactoryImpl();
    SecondaryProducer *sp;

    try
    {
        TimeInterval ti(terminationInterval, Units::SECONDS);
        StringList voNames;

        if (producerType == "LATEST")
        {
            ProducerProperties props(Storage::DATABASE,
                                     ProducerProperties::LATEST);
            sp = pfactory->createSecondaryProducer(ti, props, voNames);
        }
        else if (producerType == "HISTORY")
        {
            ProducerProperties props(Storage::DATABASE,
                                     ProducerProperties::HISTORY);
            sp = pfactory->createSecondaryProducer(ti, props, voNames);
        }
        else
        {
            std::cerr << "Invalid producer type (" << producerType << ") in test script.\n";
            exit(1);
        }

	endpoint = sp->getEndpoint();

        std::string predicate(""); // must be empty for Secondary Producer
	TimeInterval hrp(historyRP, Units::SECONDS);
	sp->declareTable(tableName, predicate, hrp);
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to create secondary producer."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    std::cout << "URL=" << endpoint.getUrl() << "\n";
    std::cout << "resourceId=" << endpoint.getResourceId() << "\n";
    std::cout << "startTime=" << startTime << "\n";
    std::cout << "endTime=" << time(NULL) << "\n";

    /* Clean up. */

    delete pfactory;
    exit(0);
}

/* End of file. */
