#! /bin/sh

# Initialise the test environment (the init script needs the absolute path
# of the test source directory).

here=`pwd`;cd `dirname $0`;test_srcdir=`pwd`;cd $here;unset here
. $test_srcdir/lib/testinit.sh $test_srcdir 10

TEST_OUTFILE=test.out
cd $TEST_DIR

TEST10_retentionPeriod=`TEST_GETPROPERTY $TEST_PROPFILE TEST10_retentionPeriod`
TEST10_marginForError=`TEST_GETPROPERTY $TEST_PROPFILE TEST10_marginForError`

# Compile test program (the compilation step is part of the test).

echo "Compiling..."

TEST_COMPILE_RGMA $TEST_SRC_PP $TEST_DIR/$TEST_EXE_PP
TEST_COMPILE_RGMA $TEST_SRC_C $TEST_DIR/$TEST_EXE_C
TEST_COMPILE_RGMA $TEST_SRC_SHOWSIGNOFLIFE $TEST_DIR/$TEST_EXE_SHOWSIGNOFLIFE

# Run the test programs. Create a producer, add a tuple with a given retention
# period; create a consumer after that retention period has almost expired and
# check the tuple remains. Wait for longer than the retention period and check
# it has expired (a LATEST query is not supposed to return expired tuples). Then
# repeat with ShowSignOfLife()s.

echo "Running..."

queryType="LATEST"
terminationInterval=$TEST10_retentionPeriod
historyRP=$TEST10_retentionPeriod
latestRP=$TEST10_retentionPeriod
firstTupleId=1
lastTupleId=1
numTuples=1

$TEST_RUN $TEST_DIR/$TEST_EXE_PP $TEST_ID \
                                 $TEST_tableName \
                                 $terminationInterval \
                                 $historyRP \
                                 $latestRP \
                                 $queryType \
                                 $firstTupleId \
                                 $lastTupleId > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_PP reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Wait for a few seconds less than the retention period.

$TEST_SLEEP `$TEST_EXPR $TEST10_retentionPeriod - $TEST10_marginForError`

# Check the tuple is still there.

$TEST_RUN $TEST_DIR/$TEST_EXE_C $TEST_ID \
                                $TEST_tableName \
                                $TEST_terminationInterval \
                                $TEST_queryTimeout \
                                $queryType \
                                $numTuples > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_C reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Wait until the retention period has expired.

$TEST_SLEEP `$TEST_EXPR 2 '*' $TEST10_marginForError`

# Check the tuple has now gone.

$TEST_RUN $TEST_DIR/$TEST_EXE_C $TEST_ID \
                                $TEST_tableName \
                                $TEST_terminationInterval \
                                $TEST_queryTimeout \
                                $queryType \
                                0 > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_C reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Now repeat, with a showSignOfLife to preserve the tuple (this means that
# we're trying to show that the terminationInterval doesn't clobber the
# producer, so we make the RP's much longer than the terminationInterval).

terminationInterval=$TEST10_retentionPeriod
historyRP=`$TEST_EXPR 2 '*' $TEST10_retentionPeriod`
latestRP=`$TEST_EXPR 2 '*' $TEST10_retentionPeriod`

$TEST_RUN $TEST_DIR/$TEST_EXE_PP $TEST_ID \
                                 $TEST_tableName \
                                 $terminationInterval \
                                 $historyRP \
                                 $latestRP \
                                 $queryType \
                                 $firstTupleId \
                                 $lastTupleId > $TEST_OUTFILE
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_PP reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

url=`TEST_GETPROPERTY $TEST_OUTFILE URL`
resourceId=`TEST_GETPROPERTY $TEST_OUTFILE resourceId`

# Wait for a few seconds less than the retention period, as before.

$TEST_SLEEP `$TEST_EXPR $TEST10_retentionPeriod - $TEST10_marginForError`

# This time, send a showSignOfLife.

$TEST_RUN $TEST_DIR/$TEST_EXE_SHOWSIGNOFLIFE "$url" \
                                             "$resourceId" \
                                             "PrimaryProducer" > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_SHOWSIGNOFLIFE reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Check the tuple is still there.

$TEST_RUN $TEST_DIR/$TEST_EXE_C $TEST_ID \
                                $TEST_tableName \
                                $TEST_terminationInterval \
                                $TEST_queryTimeout \
                                $queryType \
                                $numTuples > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_C reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Clean up.

cd $TEST_SRCDIR
$TEST_RM -rf $TEST_DIR
echo $TEST_SUCCESS_MESSAGE
exit $TEST_SUCCESS

# End of file.
