#! /bin/sh

# Initialise the test environment (the init script needs the absolute path
# of the test source directory).

here=`pwd`;cd `dirname $0`;test_srcdir=`pwd`;cd $here;unset here
. $test_srcdir/lib/testinit.sh $test_srcdir 12

TEST_OUTFILE=test.out
cd $TEST_DIR

# Compile test program (the compilation step is part of the test).

echo "Compiling..."

TEST_COMPILE_RGMA $TEST_SRC_SELECT $TEST_DIR/$TEST_EXE_SELECT

# Loop through all requested tables, reporting errors (too few or too old
# tuples) as we find them, then reporting an overall test status.

echo "Running..."

queryType="LATEST"
tables=`TEST_GETPROPERTY $TEST_PROPFILE TEST12_tables`
status=$TEST_SUCCESS
precNumTuples=0 # Number of tuples in preceding table

for table in $tables
do
	echo "Checking $table table..."

	maxAge=`TEST_GETPROPERTY $TEST_PROPFILE TEST12_maxAge_${table}`
	minTuples=`TEST_GETPROPERTY $TEST_PROPFILE TEST12_minTuples_${table}`
	select="SELECT * FROM $table"
	tableStatus=$TEST_SUCCESS

	$TEST_RUN $TEST_DIR/$TEST_EXE_SELECT $TEST_terminationInterval \
                                             $TEST_queryTimeout \
                                             $queryType \
                                             "$select" > $TEST_OUTFILE
	if [ $? -ne 0 ]
	then
		echo "Test program $TEST_EXE_SELECT reported failure."
		echo $TEST_FAILURE_MESSAGE
		exit $TEST_FAILURE
	fi

	numTuples=`TEST_GETPROPERTY $TEST_OUTFILE numTuples`
	maxTupleAge=`TEST_GETPROPERTY $TEST_OUTFILE maxTupleAge`

	echo "Found $numTuples tuples, the oldest was $maxTupleAge seconds."

	if [ "$minTuples" = "PREC" ]
	then
		if [ "$numTuples" -ne "$precNumTuples" ]
		then
			echo "(expected exactly $precNumTuples tuples)"
			tableStatus=$TEST_FAILURE
		fi
	else
		if [ "$numTuples" -lt "$minTuples" ]
		then
			echo "(expected at least $minTuples tuples)"
			tableStatus=$TEST_FAILURE
		fi
	fi

	if [ "$maxTupleAge" -gt "$maxAge" ]
	then
		echo "(expected nothing older than $maxAge seconds). "
		tableStatus=$TEST_FAILURE
	fi

	if [ $tableStatus = $TEST_FAILURE ]
	then
		status=$TEST_FAILURE # overall test status
	else
		echo "(looks OK)"
	fi

	precNumTuples=$numTuples
done

# Check overall status.

if [ $status -eq $TEST_FAILURE ]
then
	echo "The test found problems with some of the tables (see above)."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Clean up.

cd $TEST_SRCDIR
$TEST_RM -rf $TEST_DIR
echo $TEST_SUCCESS_MESSAGE
exit $TEST_SUCCESS

# End of file.
