#! /bin/sh

# Initialise the test environment (the init script needs the absolute path
# of the test source directory).
                                                                                
here=`pwd`;cd `dirname $0`;test_srcdir=`pwd`;cd $here;unset here
. $test_srcdir/lib/testinit.sh $test_srcdir 4B

TEST_EXE_CURRENTTIME=currenttime
TEST_OUTFILE=test.out
cd $TEST_DIR

# Compile test program (the compilation step is part of the test).

echo "Compiling..."

TEST_COMPILE_UTIL $TEST_SRC_CURRENTTIME $TEST_DIR/$TEST_EXE_CURRENTTIME
TEST_COMPILE_RGMA $TEST_SRC_PP $TEST_DIR/$TEST_EXE_PP
TEST_COMPILE_RGMA $TEST_SRC_C $TEST_DIR/$TEST_EXE_C

# Run the test programs (LATEST PrimaryProducer and Consumer).

testStartTime=`$TEST_DIR/$TEST_EXE_CURRENTTIME`

echo "Running..."

queryType="LATEST"
firstTupleId=1
lastTupleId=$TEST_numTuples
numTuples=$TEST_numTuples
queryTimeout=`$TEST_EXPR $TEST_queryTimeout + $numTuples / $TEST_tupleFactor`

$TEST_RUN $TEST_DIR/$TEST_EXE_PP $TEST_ID \
                                 $TEST_tableName \
                                 $TEST_terminationInterval \
                                 $TEST_historyRP \
                                 $TEST_latestRP \
                                 $queryType \
                                 $firstTupleId \
                                 $lastTupleId > $TEST_OUTFILE
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_PP reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

startTime=`TEST_GETPROPERTY $TEST_OUTFILE startTime`
endTime=`TEST_GETPROPERTY $TEST_OUTFILE endTime`
producerElapsedTime=`$TEST_EXPR $endTime - $startTime`

$TEST_SLEEP $queryTimeout # A LATEST query may return before all tuples arrive.

$TEST_RUN $TEST_DIR/$TEST_EXE_C $TEST_ID \
                                $TEST_tableName \
                                $TEST_terminationInterval \
                                $queryTimeout \
                                $queryType \
                                $numTuples > $TEST_OUTFILE
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_C reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

startTime=`TEST_GETPROPERTY $TEST_OUTFILE startTime`
endTime=`TEST_GETPROPERTY $TEST_OUTFILE endTime`
consumerElapsedTime=`$TEST_EXPR $endTime - $startTime`

testEndTime=`$TEST_DIR/$TEST_EXE_CURRENTTIME`

# Report (check?) elapsed times.

rgmaElapsedTime=`$TEST_EXPR $producerElapsedTime + $consumerElapsedTime`
testElapsedTime=`$TEST_EXPR $testEndTime - $testStartTime`

echo "Total test time: $testElapsedTime seconds."
echo "Time for R-GMA components: $rgmaElapsedTime seconds."

# Clean up.

cd $TEST_SRCDIR
$TEST_RM -rf $TEST_DIR
echo $TEST_SUCCESS_MESSAGE
exit $TEST_SUCCESS

# End of file.
