#! /bin/sh

# Initialise the test environment (the init script needs the absolute path
# of the test source directory).
                                                                                
here=`pwd`;cd `dirname $0`;test_srcdir=`pwd`;cd $here;unset here
. $test_srcdir/lib/testinit.sh $test_srcdir 5

TEST_EXE_CHECKSTATUS=checkstatus
TEST_JOB=test.job
TEST_JDL=test.jdl
TEST_JOBIDFILE=test.id
TEST_JOBSTATUSFILE=test.stat
TEST_WMSLOG=test.wmslog

cd $TEST_DIR

# Compile test program (the compilation step is part of the test).

echo "Compiling..."

TEST_COMPILE_UTIL $TEST_SRC_CHECKSTATUS $TEST_DIR/$TEST_EXE_CHECKSTATUS
TEST_COMPILE_RGMA $TEST_SRC_PP $TEST_DIR/$TEST_EXE_PP
TEST_COMPILE_RGMA $TEST_SRC_C $TEST_DIR/$TEST_EXE_C

# Run the test programs (HISTORY PrimaryProducer on Worker Node and
# Consumer on UI Node).

echo "Running..."

queryType="HISTORY"
firstTupleId=1
lastTupleId=$TEST_numTuples
numTuples=$TEST_numTuples
queryTimeout=`$TEST_EXPR $TEST_queryTimeout + $numTuples / $TEST_tupleFactor`

# Create JDL and shell script to submit to Grid by passing the template files
# through sed. Then submit them to the Grid (to create the producer).

$TEST_SED -e "s#@TEST_SH@#${TEST_JOB}#g
              s#@TEST_EXE@#${TEST_EXE_PP}#g
              s#@TEST_DIR@#${TEST_DIR}#g" $TEST_JDL_TEMPLATE > $TEST_JDL

$TEST_CP $TEST_FUNCTIONS $TEST_JOB
$TEST_SED -e "s#@TEST_EXE@#${TEST_EXE_PP}#g
              s#@TEST_ID@#${TEST_ID}#g
              s#@TEST_RUN@#${TEST_RUN}#g
              s#@TEST_tableName@#${TEST_tableName}#g
              s#@TEST_terminationInterval@#${TEST_terminationInterval}#g
              s#@TEST_historyRP@#${TEST_historyRP}#g
              s#@TEST_latestRP@#${TEST_latestRP}#g
              s#@TEST_queryType@#${queryType}#g
              s#@TEST_firstTupleId@#${firstTupleId}#g
              s#@TEST_lastTupleId@#${lastTupleId}#g
	      s#@TEST_FAILURE_MESSAGE@#${TEST_FAILURE_MESSAGE}#g
	      s#@TEST_FAILURE@#${TEST_FAILURE}#g" $TEST_PP_JOB_TEMPLATE >>  $TEST_JOB

# Submit the producer job to the Grid for processing.

$TEST_JOBSUBMIT -o $TEST_JOBIDFILE $TEST_DIR/$TEST_JDL > $TEST_WMSLOG 2>&1
if [ $? -ne 0 ]
then
	$TEST_CAT $TEST_WMSLOG
	echo "Unable to submit $TEST_JDL to Grid."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi
echo "Submitted job `$TEST_AWK '/^[^#]/' $TEST_JOBIDFILE`..."

# Wait for producer job to complete (but time out eventually).

timeLeft=$TEST_jobWaitTime # seconds
while :
do
	echo "Waiting for Primary Producer job to complete..."
	$TEST_SLEEP $TEST_jobPollTime

	$TEST_JOBSTATUS -i $TEST_JOBIDFILE > $TEST_JOBSTATUSFILE 2> $TEST_WMSLOG
	if [ $? -ne 0 ]
	then
		$TEST_CAT $TEST_WMSLOG
		echo "Unable to determine job status."
		echo $TEST_FAILURE_MESSAGE
		exit $TEST_FAILURE
	fi

	$TEST_DIR/$TEST_EXE_CHECKSTATUS $TEST_JOBSTATUSFILE
	jobStatus=$?
	if [ $jobStatus -eq $TEST_SUCCESS ]
	then
		break # job has completed
	elif [ $jobStatus -eq $TEST_FAILURE -o $jobStatus -eq $TEST_ABANDON ]
	then
		echo "Grid job failed."
		$TEST_CAT $TEST_JOBSTATUSFILE
		echo $TEST_FAILURE_MESSAGE
		exit $TEST_FAILURE
	else
		# keep waiting
		timeLeft=`$TEST_EXPR $timeLeft - $TEST_jobPollTime`
		if [ $timeLeft -le 0 ]
		then
			$TEST_JOBCANCEL -i $TEST_JOBIDFILE >/dev/null 2>&1
			echo "Timed out waiting for job to complete."
			echo $TEST_FAILURE_MESSAGE
			exit $TEST_FAILURE
		fi
	fi
done

# Retrieve the job output from the CE. For now, this is just to clean up the
# job, but as in the test4A/B/C tests, you can extract information (e.g. times)
# from program's standard output, using the TEST_GETPROPERTY function.

$TEST_JOBOUTPUT -i $TEST_JOBIDFILE --dir $TEST_DIR > $TEST_WMSLOG 2>&1
if [ $? -ne 0 ]
then
	$TEST_CAT $TEST_WMSLOG
	echo "Can't retrieve job output from CE."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Create a Consumer locally, to check the tuples arrived.

echo "Checking tuples..."

$TEST_RUN $TEST_DIR/$TEST_EXE_C $TEST_ID \
                                $TEST_tableName \
                                $TEST_terminationInterval \
                                $queryTimeout \
                                $queryType \
                                $numTuples > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_C reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Clean up.

cd $TEST_SRCDIR
$TEST_RM -rf $TEST_DIR
echo $TEST_SUCCESS_MESSAGE
exit $TEST_SUCCESS

# End of file.
