#! /bin/sh

# Initialise the test environment (the init script needs the absolute path
# of the test source directory).
                                                                                
here=`pwd`;cd `dirname $0`;test_srcdir=`pwd`;cd $here;unset here
. $test_srcdir/lib/testinit.sh $test_srcdir 8

TEST_EXE_CHECKSTATUS=checkstatus
TEST_JOB=test.job
TEST_JDL=test.jdl
TEST_JOBIDFILE=test.id
TEST_JOBSTATUSFILE=test.stat
TEST_WMSLOG=test.wmslog

cd $TEST_DIR

# Compile test program (the compilation step is part of the test).

echo "Compiling..."

TEST_COMPILE_UTIL $TEST_SRC_CHECKSTATUS $TEST_DIR/$TEST_EXE_CHECKSTATUS
TEST_COMPILE_RGMA $TEST_SRC_PP $TEST_DIR/$TEST_EXE_PP
TEST_COMPILE_RGMA $TEST_SRC_C $TEST_DIR/$TEST_EXE_C

# Run the test programs (HISTORY PrimaryProducer on UI and Consumer on the
# Worker Node).

echo "Running..."

terminationInterval=`$TEST_EXPR $TEST_terminationInterval + $TEST_jobWaitTime + $TEST_jobPollTime`
historyRP=`$TEST_EXPR $TEST_historyRP + $TEST_jobWaitTime + $TEST_jobPollTime`
latestRP=`$TEST_EXPR $TEST_latestRP + $TEST_jobWaitTime + $TEST_jobPollTime`
queryType="HISTORY"
firstTupleId=1
lastTupleId=$TEST_numTuples
numTuples=$TEST_numTuples
queryTimeout=`$TEST_EXPR $TEST_queryTimeout + $numTuples / $TEST_tupleFactor`

# Create Primary Producer locally (its tuples must hang around until the job
# has completed, hence the extended terminationInterval and RPs)...

$TEST_RUN $TEST_DIR/$TEST_EXE_PP $TEST_ID \
                                 $TEST_tableName \
                                 $terminationInterval \
                                 $historyRP \
                                 $latestRP \
                                 $queryType \
                                 $firstTupleId \
                                 $lastTupleId > /dev/null
if [ $? -ne 0 ]
then
	echo "Test program $TEST_EXE_PP reported failure."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Create JDL and shell script to submit to Grid by passing the template files
# through sed. Then submit them to the Grid (to create the Consumer).

$TEST_SED -e "s#@TEST_SH@#${TEST_JOB}#g
              s#@TEST_EXE@#${TEST_EXE_C}#g
              s#@TEST_DIR@#${TEST_DIR}#g" $TEST_JDL_TEMPLATE > $TEST_JDL

$TEST_CP $TEST_FUNCTIONS $TEST_JOB
$TEST_SED -e "s#@TEST_EXE@#${TEST_EXE_C}#g
              s#@TEST_ID@#${TEST_ID}#g
              s#@TEST_RUN@#${TEST_RUN}#g
              s#@TEST_tableName@#${TEST_tableName}#g
              s#@TEST_terminationInterval@#${TEST_terminationInterval}#g
              s#@TEST_queryTimeout@#${queryTimeout}#g
              s#@TEST_queryType@#${queryType}#g
              s#@TEST_numTuples@#${numTuples}#g
	      s#@TEST_FAILURE_MESSAGE@#${TEST_FAILURE_MESSAGE}#g
	      s#@TEST_FAILURE@#${TEST_FAILURE}#g" $TEST_C_JOB_TEMPLATE >>  $TEST_JOB

# Submit the consumer jobs to the Grid for processing.

$TEST_RM -f $TEST_JOBIDFILE # all Job IDs will go into this file

echo "Submitting $TEST_numJobs Consumer jobs..."
i=0
while [ $i -lt $TEST_numJobs ]
do
	i=`$TEST_EXPR $i + 1`

	$TEST_JOBSUBMIT -o $TEST_JOBIDFILE $TEST_DIR/$TEST_JDL > $TEST_WMSLOG 2>&1
	if [ $? -ne 0 ]
	then
		$TEST_CAT $TEST_WMSLOG
		echo "Unable to submit $TEST_JDL to Grid."
		echo $TEST_FAILURE_MESSAGE
		exit $TEST_FAILURE
	fi
done

jobsToWaitFor=`$TEST_AWK '/^[^#]/' $TEST_JOBIDFILE`

# Wait for consumer jobs to complete (but time out eventually).

timeLeft=$TEST_jobWaitTime # seconds
while :
do
        echo "Waiting for Consumer jobs to complete..."
	$TEST_SLEEP $TEST_jobPollTime

	jobsStillNotFinished=""
	for jobId in $jobsToWaitFor
	do
		$TEST_JOBSTATUS "$jobId" > $TEST_JOBSTATUSFILE 2> $TEST_WMSLOG
		if [ $? -ne 0 ]
		then
			$TEST_JOBCANCEL --noint -i $TEST_JOBIDFILE > /dev/null 2>&1
			$TEST_CAT $TEST_WMSLOG
			echo "Unable to determine job status for $jobId."
			echo $TEST_FAILURE_MESSAGE
			exit $TEST_FAILURE
		fi
	
		$TEST_DIR/$TEST_EXE_CHECKSTATUS $TEST_JOBSTATUSFILE
		jobStatus=$?
		if [ $jobStatus -eq $TEST_SUCCESS ]
		then
			echo "Job $jobId is complete."
		elif [ $jobStatus -eq $TEST_FAILURE -o $jobStatus -eq $TEST_ABANDON ]
		then
			$TEST_JOBCANCEL --noint -i $TEST_JOBIDFILE > /dev/null 2>&1
			$TEST_CAT $TEST_JOBSTATUSFILE
			echo "Grid job failed."
			echo $TEST_FAILURE_MESSAGE
			exit $TEST_FAILURE
		elif [ $jobStatus -eq $TEST_WAIT ]
		then
			jobsStillNotFinished="$jobsStillNotFinished $jobId"
		fi
	done # (this jobID)

	# See if we have anything left to wait for.

	jobsToWaitFor=`echo $jobsStillNotFinished` # get rid of padding
	if [ -z "$jobsToWaitFor" ]
	then
		break # all jobs complete
	fi

	timeLeft=`$TEST_EXPR $timeLeft - $TEST_jobPollTime`
	if [ $timeLeft -le 0 ]
	then
		$TEST_JOBCANCEL --noint -i $TEST_JOBIDFILE > /dev/null 2>&1
		echo "Timed out waiting for jobs to complete."
		echo $TEST_FAILURE_MESSAGE
		exit $TEST_FAILURE
	fi

done # (all jobs)

# Retrieve each job's output from the CE. For now, this is just to clean up the
# job, but as in the test4A/B/C tests, you can extract information (e.g. times)
# from program's standard output, using the TEST_GETPROPERTY function. Each
# job's output is put in its own subdirectory.

$TEST_JOBOUTPUT --dir $TEST_DIR --noint -i $TEST_JOBIDFILE > $TEST_WMSLOG 2>&1
if [ $? -ne 0 ]
then
	$TEST_CAT $TEST_WMSLOG
	echo "Can't retrieve output from jobs."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Clean up.

cd $TEST_SRCDIR
$TEST_RM -rf $TEST_DIR
echo $TEST_SUCCESS_MESSAGE
exit $TEST_SUCCESS

# End of file.
